package net.grelf.grip;

// This Java source file is part of the application GRIP, for processing and
// measuring digital photographs.
// GRIP is available free from www.grelf.net - please always quote this URL in
// connection with GRIP.
//
// Copyright (c) Graham T Relf, UK, 2006
//
// All software contains bugs, even if, we hope, they are only extremely subtle
// ones. The author has tested this software to the best of his abilities to
// ensure that it is generally fit for its purpose of processing and measuring
// digital photographs. The author makes absolutely no claims and gives no
// guarantees whatsoever as to the accuracy of any output from this software.
// Anyone making decisions based on any output from this software has a
// responsibility to first convince themselves of the appropriateness of so
// doing. They should test the software for themselves and understand how it
// works, sufficiently for their needs. Source code and explanatory
// documentation are available for that purpose.

/** Blob menu (on each BlobFrame that has been opened by clicking on a
  * detected blob) and its actions. */
public class BlobMenu
extends javax.swing.JMenu
implements java.awt.event.ActionListener, javax.swing.event.MenuListener
{
	private ImFrame imframe;
  private BlobMeas meas;
	private javax.swing.JMenuItem /* itemView, itemMeasure, itemAdjust, */
                                /* itemId, itemMagnitude, itemClear, itemPaste, */
					                      /* itemPosition, */ itemBands, /* itemEstimate, */
                                 itemTable, itemSave;

	/** Set when a Star is pasted onto any blob. Used to display the MagnitudeImage. */
	private static net.grelf.astro.StarChart chart = null;

	private static net.grelf.astro.MagnitudeBands magnitudeBands = null;

	/** Enable/disable menu options according to the number of channels and bits
    * per channel in the currently displayed image. */
	public void reviseSettings ()
	{
//		itemView.setEnabled (true);

		if (null != this.meas.blob.getBackgroundDisc ()
		 && null != this.meas.blob.getBrightnessDisc ())
		{
//			itemAdjust.setEnabled (true);
//			itemMeasure.setEnabled (true);
		}
		else
		{
//			itemAdjust.setEnabled (false);
//			itemMeasure.setEnabled (false);
		}

//		itemId.setEnabled (true);
//		itemMagnitude.setEnabled (true);
		itemTable.setEnabled (true);

    if (null != this.meas)
    {
      net.grelf.astro.Star star = this.meas.getStar ();
  //		itemClear.setEnabled (null != star);
      itemBands.setEnabled (null != star);
/*
      if (null != star
       && 0 < star.getId ().length ()
       && null == star.getMagnitude ()) // Unknown magnitude
      {
        itemEstimate.setEnabled (true);
      }
      else
      {
        itemEstimate.setEnabled (false);
      }
*/
/*
      if (null != star
       && 0 < star.getId ().length ()
       && null == star.getPosition ()) // Unknown RA/Dec
      {
        itemPosition.setEnabled (true);
      }
      else
      {
        itemPosition.setEnabled (false);
      }
*/
    }
    else
    {
      itemBands.setEnabled (false);
//      itemEstimate.setEnabled (false);
//      itemPosition.setEnabled (false);
    }

//		itemPaste.setEnabled (null != net.grelf.astro.StarChart.getSelectedStar ());
		itemSave.setEnabled (true); // If we got this menu there must be some blobs.
	} // reviseSettings

	public BlobMenu (BlobFrame frame)
	{
		super ("Blobs/stars");
		this.imframe = frame;
		this.meas = frame.getBlobMeas ();
    setMnemonic ('B');
    initialise ();
  } // BlobMenu

  private void initialise ()
  {
		int nBands = this.imframe.getParentFrame ().getImage ().getNBands ();

		if (null == magnitudeBands									// First time blob menu is shown
		 || magnitudeBands.getNBands () != nBands)  // Working on a different image
		{
			magnitudeBands = new net.grelf.astro.MagnitudeBands (nBands);
		}

//		add (itemView = net.grelf.Util.menuItem (this, "View data for this blob/star", "view", 'V', 0));
//		add (itemAdjust = net.grelf.Util.menuItem (this, "Adjust measuring circles", "adjust", 'J', 0));
//		itemAdjust.setToolTipText ("Star must be entirely in inner disc. Outer ring pure background.");
//		add (itemMeasure = net.grelf.Util.menuItem (this, "Measure brightness using the circles", "measure", 'B', 0));
//		itemMeasure.setToolTipText ("Much more accurate than initially detected brightness");
// TODO Accuracy of brightness remeasurement
/****
		add (itemRemeasure = net.grelf.Util.menuItem (this, "Remeasure all stars using the circles", "remeasure", 'R', 0));
		itemRemeasure.setToolTipText ("Remeasure brightnesses of all identified stars");
****/
//		add (itemId = net.grelf.Util.menuItem (this, "Identify this blob as a star", "id", 'I', 0));
//		itemId.setToolTipText ("Star to be estimated/positioned must have id but not magnitude/position");
//		add (itemMagnitude = net.grelf.Util.menuItem (this, "Enter known magnitude for this blob", "magnitude", 'M', 0));
//		itemMagnitude.setToolTipText ("If not getting magnitude(s) by pasting star data");
//		add (itemPaste = net.grelf.Util.menuItem (this, "Paste data from star chart", "paste", 'P', java.awt.event.KeyEvent.VK_V));
//		itemPaste.setToolTipText ("Must have created a GRIP star chart and clicked on a star");
//		add (itemClear = net.grelf.Util.menuItem (this, "Clear star data", "clear", 'C', 0));
		add (itemBands = net.grelf.Util.menuItem (this,
            "Set wavelength bands for estimating", "bands", 'W', 0));
/*
		add (itemEstimate = net.grelf.Util.menuItem (this,
            "Estimate magnitude from other blobs", "estimate", 'E', 0));
		itemEstimate.setToolTipText (
            "Estimates all blobs/stars with id but no magnitude");
		add (itemPosition = net.grelf.Util.menuItem (this,
            "Measure position from reference stars", "position",
            'R', 0));
		itemPosition.setToolTipText (
            "Measures all blobs/stars with id but no RA/Dec");
*/
    add (itemTable = net.grelf.Util.menuItem (this,
            "Tabulate identified blobs", "table", 'T', 0));
		add (itemSave = net.grelf.Util.menuItem (this,
            "Save all blob/star data", "save", 'S', 0));
		itemSave.setToolTipText (
            "Save parent image\'s blob data as XML .blobs file");
		addSeparator ();
		add (net.grelf.Util.menuItem (this,
            "Help about this menu", "help", 0, 0));

		addMenuListener (this); // Need to be able to revise settings when the menu is invoked

		reviseSettings ();
	} // initialise

	/** Implementing the ActionListener interface to act on menu option
    * selection. */
	@Override
	public void actionPerformed (java.awt.event.ActionEvent ev)
	{
		String action = ev.getActionCommand ();
    switch (action)
    {
      case "bands":
        magnitudeBands.showDialogue (this.imframe);
        break;

      case "estimate":
        try
        {
          ImFrame parentFrame = this.imframe.getParentFrame ();
          BlobMeasList measList = parentFrame.getBlobMeases ();
          net.grelf.astro.MagnitudeGraph graph =
            new net.grelf.astro.MagnitudeGraph (
              this.imframe, "Stellar brightnesses v. magnitudes",
              measList, magnitudeBands, parentFrame.getFilePath (),
              parentFrame.getImageTimeSpan ());

          // Try to show a copy of the chart with the measured stars marked in it:
          new net.grelf.astro.MagnitudeChart (chart, "Measured stars", measList,
                                              graph.isAccurate ());

          // Then show the graph (modal):
          graph.showGraph ();
        }
        catch (net.grelf.InadequateDataException ex)
        {
          net.grelf.Util.warning ("Sorry", ex.getMessage ());
        }

        break;

      case "help":
        HelpMenu.help ("pages/blob_menu.html");
        break;

      case "position":
        net.grelf.astro.Star star = this.meas.getStar ();

        if (null != star)
        {
          ImFrame parentFrame = this.imframe.getParentFrame ();
          BlobMeasList measList = parentFrame.getBlobMeases ();
          net.grelf.astro.SkyPoint pt =
                  star.measurePosition (parentFrame, measList);

          if (null == pt)
          {
            net.grelf.Util.warning (parentFrame, "Measured position",
                    "Sorry, not measured");
          }
          else
          {
            javax.swing.JTextArea text =
                    new javax.swing.JTextArea (pt.toString ());
            net.grelf.Util.message (parentFrame,
                    "Measured position (copyable text)", text);
          }
        }

        break;

      case "save":
        this.imframe.getParentFrame ().saveBlobMeasList ();
        break;

      case "table":
        BlobMeasList measList = this.imframe.getParentFrame ().getBlobMeases ();
        new net.grelf.astro.MagnitudeTable (this.imframe, "Identified blobs",
                measList).showTable ();
        break;
    }

    reviseSettings ();
	} // actionPerformed

	/** Calls reviseSettings (). Implementing MenuListener. */
	@Override
	public void menuSelected (javax.swing.event.MenuEvent ev)
	{
		reviseSettings ();
	} // menuSelected

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuDeselected (javax.swing.event.MenuEvent ev) {}

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuCanceled (javax.swing.event.MenuEvent ev) {}

} // BlobMenu