package net.grelf.grip;

// This Java source file is part of the application GRIP, for processing and
// measuring digital photographs.
// GRIP is available free from www.grelf.net - please always quote this URL in
// connection with GRIP.
//
// Copyright (c) Graham T Relf, UK, 2006
//
// All software contains bugs, even if, we hope, they are only extremely subtle
// ones. The author has tested this software to the best of his abilities to
// ensure that it is generally fit for its purpose of processing and measuring
// digital photographs. The author makes absolutely no claims and gives no
// guarantees whatsoever as to the accuracy of any output from this software.
// Anyone making decisions based on any output from this software has a
// responsibility to first convince themselves of the appropriateness of so
// doing. They should test the software for themselves and understand how it
// works, sufficiently for their needs. Source code and explanatory
// documentation are available for that purpose.

/** This is an ImFrame menu for experimental programming - not for general users. */
public class ExperimentMenu
extends javax.swing.JMenu
implements java.awt.event.ActionListener
{
  private final ImFrame imframe;

  public ExperimentMenu (ImFrame imf)
  {
    super ("Experiment");
    this.imframe = imf;
    initialise ();
  } // ExperimentMenu

  private void initialise ()
  {
    setMnemonic ('X');
    add (net.grelf.Util.menuItem (this, "Green to red", "gr", 'G', 0));
    add (net.grelf.Util.menuItem (this, "Blue to red", "br", 'B', 0));
    add (net.grelf.Util.menuItem (this, "Draw colour cube sides", "cube", 'C', 0));
    add (net.grelf.Util.menuItem (this, "Draw Lyapunov curve", "lyap", 'L', 0));
		add (net.grelf.Util.menuItem (this, "Hough line detector", "hough", 'H', 0));
    add (net.grelf.Util.menuItem (this, "Zoom effect (mark point)", "zoom", 'Z', 0));
    add (net.grelf.Util.menuItem (this, "Swirl effect (about centre)", "swirl", 'S', 0));
    add (net.grelf.Util.menuItem (this, "Recolour burnt-out stars", "recolour", 'R', 0));
    add (net.grelf.Util.menuItem (this, "Segment non-green", "nongreen", 'N', 0));
    add (net.grelf.Util.menuItem (this, "Recolour non-green", "regreen", 'E', 0));
  } // initialise

  @Override
  public void actionPerformed (java.awt.event.ActionEvent ev)
  {
		String action = ev.getActionCommand ();

    switch (action)
    {
      case "br": blueToRed (); break;
      case "cube": new ImGraphic ().drawColourCubeSides (); break;
      case "gr": greenToRed (); break;
      case "nongreen": segmentNonGreen (); break;
      case "hough": houghLines (); break;
      case "lyap": new ImGraphic ().drawLyapunov (); break;
      case "recolour": recolourStars (); break;
      case "regreen": recolourNonGreen (); break;
      case "swirl": swirlEffect (); break;
      case "zoom": zoomEffect (); break;
    }
  } // actionPerformed

  private void blueToRed ()
  {
    net.grelf.image.Image im = this.imframe.getImage ();
    int nb = im.getNBands ();

    if (3 != nb)
    {
      net.grelf.Util.warning (imframe,
              "Sorry", "Operation is only for RGB images");
      return;
    }

    this.imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
    int wd = im.getWidth ();
    int ht = im.getHeight ();
    int [] px;

    for (int y = 0; y < ht; y++)
    {
      for (int x = 0; x < wd; x++)
      {
        px = im.getPixel (x, y);

        if (px [2] > px [0] && px [2] > px [1]) // Predominantly blue
        {
          // Swap blue and red:
          int b = px [2];
          px [2] = px [0];
          px [0] = b;
        }

        im.setPixel (x, y, px);
      }
    }

    this.imframe.redisplay ();
		this.imframe.setCursor (this.imframe.IMFRAME_DEFAULT_CURSOR);
  } // blueToRed

  private void greenToRed ()
  {
    net.grelf.image.Image im = this.imframe.getImage ();
    int nb = im.getNBands ();

    if (3 != nb)
    {
      net.grelf.Util.warning (imframe,
              "Sorry", "Operation is only for RGB images");
      return;
    }

    this.imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
    int wd = im.getWidth ();
    int ht = im.getHeight ();
    int [] px;

    for (int y = 0; y < ht; y++)
    {
      for (int x = 0; x < wd; x++)
      {
        px = im.getPixel (x, y);

        if (px [1] > px [2] && px [0] > px [2]) // Predominantly green
        {
          // Swap green and red:
          int g = px [1];
          px [1] = px [0];
          px [0] = (int) (g * 1.25); // Red stronger
        }

        im.setPixel (x, y, px);
      }
    }

    this.imframe.redisplay ();
		this.imframe.setCursor (this.imframe.IMFRAME_DEFAULT_CURSOR);
  } // greenToRed

/****/
  // Testing ByteMask.houghLines ()
  private void houghLines ()
  {
    BlobMask bm = StarSegmenter.segment (
      imframe.getImage (),
      Integer.parseInt (Config.getValue ("StarSegmentDifference.value")),
      Integer.parseInt (Config.getValue ("StarSegmentRadius.value")));
 		imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
    java.util.List <net.grelf.Line> lines = bm.houghLines ();

    if (lines.isEmpty ())
    {
      net.grelf.Util.message ("Sorry", "No lines were detected");
    }
    else
    {
      net.grelf.image.Image im = imframe.getImage ();
      int wd = im.getWidth ();
      int ht = im.getHeight ();

      for (net.grelf.Line line : lines)
      {
System.out.println ("y = " + line.getGradient () + " . x + " + line.getIntercept ());
        net.grelf.LineSegment seg = line.clip (wd, ht);
        net.grelf.PointFloat [] ends = seg.getEnds ();

        if (null == ends || ends.length != 2)
        {
          net.grelf.Util.warning ("Error", "Could not clip line to image");
        }
        else
        {
          java.awt.Point pt1 = ends [0].toPoint ();
          java.awt.Point pt2 = ends [1].toPoint ();
System.out.println (pt1.toString ());
System.out.println (pt2.toString ());

          PointList points = new PointList (new DefaultMeasurementHandler ());
          points.add (pt1);
          points.add (pt2);
          GlassPane gp = imframe.getGP ();
          gp.setPoints (points);
          gp.setDrawingMode (DrawingMode.POINT_PAIR);
          gp.repaint ();
//          imframe.redisplay ();
        }
      }
    }

    imframe.setCursor (imframe.IMFRAME_DEFAULT_CURSOR);
  } // houghLines
  
  private void segmentNonGreen ()
  {
    net.grelf.image.Image im = imframe.getImage ();
    int nBands = im.getNBands ();
    
    if (3 != nBands)
    {
      net.grelf.Util.warning (imframe, "Sorry", 
          "Operation only available for RGB images");
      return;
    }
    
 		imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
    int wd = im.getWidth ();
    int ht = im.getHeight ();
    byte [][] mask = new byte [wd][ht];
    GlassPane gp = imframe.getGP ();
    gp.setDrawingMode (DrawingMode.MASK);
    gp.setMask (new BlobMask (mask));
    int [] px;
    
    for (int y = 0; y < ht; y++)
    {
      for (int x = 0; x < wd; x++)
      {
        px = im.getPixel (x, y);
        
        if (px [0] >= px [1] || px [2] >= px [1])
        {
          mask [x][y] = (byte) 1;
        }
      }
    }
    
    gp.repaint ();
    imframe.redisplay ();
    imframe.setCursor (imframe.IMFRAME_DEFAULT_CURSOR);
  } // segmentNonGreen

  private void swirlEffect ()
  {
    int n = net.grelf.Util.askInteger (
            this.imframe, "Number of iterations? ", 10, 1, 100);

    if (n != Integer.MIN_VALUE)
    {
      double degs =
        net.grelf.Util.askDouble (this.imframe,
              "Angle (degrees) per rotation step?", 0.1, 0.00001, 5.0);

      if (!Double.isNaN (degs))
      {
        net.grelf.Angle angle =
                new net.grelf.Angle (degs, net.grelf.Angle.Units.DEGREES);
        this.imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
        boolean replaceBrighter =
                net.grelf.Util.confirm (this.imframe,
                "Chooose", "Do you wish only to replace brighter pixels?");
        try
        {
          net.grelf.image.Image im = this.imframe.getImage ();
          int wd = im.getWidth ();
          int ht = im.getHeight ();
          net.grelf.image.Image32 accum =
                  new net.grelf.image.Image32 (wd, ht, im.getNBands (), true);
                                                                 // fill with 0
          accum.add (im);

          for (int i = 0; i < n; i++)
          {
            im.rotate (angle);
            int newWd = im.getWidth ();
            int newHt = im.getHeight ();
            int dwd2 = (newWd - wd) / 2;
            int dht2 = (newHt - ht) / 2;
            java.awt.Point topLeft = new java.awt.Point (dwd2, dht2);
            java.awt.Point bottomRight =
                    new java.awt.Point (newWd - dwd2, newHt - dht2);
            im.crop (topLeft, bottomRight);

            if (replaceBrighter)
            {
              accum.replaceBrighterPixels (im);
            }
            else
            {
              accum.add (im);
            }
          }

          this.imframe.getImPane ().setImage (accum);
          this.imframe.setCursor (this.imframe.IMFRAME_DEFAULT_CURSOR);
          this.imframe.redisplay ();
        }
        catch (net.grelf.image.IncompatibleImageException ex)
        {
          net.grelf.Util.logWarning (
                  "Incompatible image exception in swirlEffect ()");
        }
      }
    }
  } // swirlEffect

  private static String savedCaption;

  private void zoomEffect ()
  {
    savedCaption = this.imframe.getCaption ();
    this.imframe.setCaption ("Point to zoom around");
    this.imframe.getGP ().setDrawingMode (DrawingMode.ZOOM_EFFECT);
  } // zoomEffect

  public static void zoomEffect (ImFrame imf, java.awt.Point centre)
  {
    imf.setCaption (savedCaption);
    imf.getGP ().setDrawingMode (DrawingMode.UNDEFINED);
    int n = net.grelf.Util.askInteger (
            imf, "Number of iterations? ", 10, 1, 100);

    if (n != Integer.MIN_VALUE)
    {
      double f = net.grelf.Util.askDouble (
              imf, "Scale factor per zoom step?", 1.01, 0.8, 1.2);

      if (!Double.isNaN (f))
      {
        imf.setCursor (GRIP.GRIP_WAIT_CURSOR);
        boolean replaceBrighter = net.grelf.Util.confirm (
                imf, "Chooose", "Do you wish only to replace brighter pixels?");
        try
        {
          net.grelf.image.Image im = imf.getImage ();
          int wd = im.getWidth ();
          int ht = im.getHeight ();
          net.grelf.image.Image32 accum =
                  new net.grelf.image.Image32 (wd, ht, im.getNBands (), true);
                                                                 // fill with 0
          accum.add (im);

          for (int i = 0; i < n; i++)
          {
            im.scale (f, true, centre);

            if (replaceBrighter)
            {
              accum.replaceBrighterPixels (im);
            }
            else
            {
              accum.add (im);
            }
          }

          imf.getImPane ().setImage (accum);
          imf.setCursor (imf.IMFRAME_DEFAULT_CURSOR);
          imf.redisplay ();
        }
        catch (net.grelf.image.IncompatibleImageException ex)
        {
          net.grelf.Util.logWarning (
                  "Incompatible image exception in zoomEffect ()");
        }
      }
    }
  } // zoomEffect

  private void recolourStars ()
  {
    net.grelf.image.Image im = imframe.getImage ();
    net.grelf.image.Image32 im32;
    
    if (im instanceof net.grelf.image.Image32)
    {
      im32 = (net.grelf.image.Image32) im;
    }
    else
    {
      im32 = (net.grelf.image.Image32) im.convertToImage32 ();
      imframe.getImPane ().setImage (im32);
    }
        
    int nBands = im32.getNBands ();

    if (nBands < 2)
    {
      net.grelf.Util.warning ("Sorry", 
          "Recolouring is not possible in monochrome images");
      return;
    }

    BlobMeasList blobMeases = imframe.getBlobMeases ();
    
    if (null == blobMeases)
    {
      try
      {
        StarSegmenter.segment (imframe, Integer.parseInt (
                Config.getValue ("StarSegmentDifference.value")),
                                        Integer.parseInt (
                Config.getValue ("StarSegmentRadius.value")),
                true); // Erode/dilate
        imframe.addHistory ("Segment stars automatically");
        boolean includeSinglePixelBlobs = false;
        imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
        blobMeases = imframe.detectBlobs (includeSinglePixelBlobs);

        if (null != blobMeases && blobMeases.size () > 0)
        {
          imframe.setBlobMeases (blobMeases);
          // Set reference in ImFrame so that measurements can be shown.

          imframe.getGP ().setDrawingMode (DrawingMode.BLOB_HOVER);
        }
        else
        {
          net.grelf.Util.message ("Sorry",
                  "No stars detected.\nParameters may need adjusting.");
        }

        imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
      }
      catch (NumberFormatException ex)
      {
        String msg = "Config values missing for StarSegmenter parameters.\n" +
                     "Try deleting file grip.properties and re-running GRIP.";
        net.grelf.Util.warning (this, "Error", msg);
        return;
      }
    }
    
    imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);

    for (BlobMeas bm : blobMeases)
    {
      bm.blob.recolourIfSaturated (im32);
    }
    
    im32.getRangeByRescanning ();
    imframe.redisplay ();
    imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
  } // recolourStars

  private void recolourNonGreen ()
  {
    net.grelf.image.Image im = imframe.getImage ();
    int nBands = im.getNBands ();
    
    if (3 != nBands)
    {
      net.grelf.Util.warning (imframe, "Sorry", 
          "Operation only available for RGB images");
      return;
    }
    
 		imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
    int wd = im.getWidth ();
    int ht = im.getHeight ();
    byte [][] mask = new byte [wd][ht];
    int [] px;
    
    for (int y = 0; y < ht; y++)
    {
      for (int x = 0; x < wd; x++)
      {
        px = im.getPixel (x, y);
        
        if (px [0] >= px [1] || px [2] >= px [1])
        {
          mask [x][y] = (byte) 1;
        }
      }
    }
    
    for (int y = 0; y < ht; y++)
    {
      for (int x = 0; x < wd; x++)
      {
        px = im.getPixel (x, y);
        
        if (mask [x][y] != (byte) 0)
        {
          int temp = px [2];
          px [2] = px [0];
          px [0] = temp;
          im.setPixel (x, y, px);
        }
      }
    }
    
    imframe.redisplay ();
    imframe.setCursor (imframe.IMFRAME_DEFAULT_CURSOR);
  } // recolourNonGreen

} // ExperimentMenu
