package net.grelf.grip;

// This Java source file is part of the application GRIP, for processing and
// measuring digital photographs.
// GRIP is available free from www.grelf.net - please always quote this URL in
// connection with GRIP.
//
// Copyright (c) Graham T Relf, UK, 2006
//
// All software contains bugs, even if, we hope, they are only extremely subtle
// ones. The author has tested this software to the best of his abilities to
// ensure that it is generally fit for its purpose of processing and measuring
// digital photographs. The author makes absolutely no claims and gives no
// guarantees whatsoever as to the accuracy of any output from this software.
// Anyone making decisions based on any output from this software has a
// responsibility to first convince themselves of the appropriateness of so
// doing. They should test the software for themselves and understand how it
// works, sufficiently for their needs. Source code and explanatory
// documentation are available for that purpose.

/** Geometry menu and its actions. */
public class GeometryMenu
extends javax.swing.JMenu
implements java.awt.event.ActionListener, javax.swing.event.MenuListener, ImageSizeAction
{
	private ImFrame imframe;
	private javax.swing.JMenuItem itemRaw;

	public GeometryMenu (ImFrame frame)
	{
		super ("Geometry");
		this.imframe = frame;
    setMnemonic ('G');
    initialise ();
  } // GeometryMenu

  private void initialise ()
  {
		add (net.grelf.Util.menuItem (this,
            "Crop", "crop", 'C', 0));
		add (net.grelf.Util.menuItem (this,
            "Auto-crop (remove black rim)", "autocrop", 'A', 0));
		add (net.grelf.Util.menuItem (this,
            "Crop to nearest powers of 2", "croppower2", '2', 0));
		add (net.grelf.Util.menuItem (this,
            "Expand (black rim) to nearest powers of 2", "expandpower2", 'X', 0));

		if (this.imframe.getImage ().isRaw ())
		{
			add (itemRaw = net.grelf.Util.menuItem (this,
              "Remove RAW margin", "margin", 'M', 0));
		}

		addSeparator ();
		add (net.grelf.Util.menuItem (this,
            "Flip horizontally", "fliph", 'H', 0));
		add (net.grelf.Util.menuItem (this,
            "Flip vertically", "flipv", 'V', 0));
		add (net.grelf.Util.menuItem (this,
            "Scale", "scale", 'S', 0));
    add (net.grelf.Util.menuItem (this,
            "Fit image in m x n", "fit", 'F', java.awt.event.KeyEvent.VK_F));
		add (net.grelf.Util.menuItem (this,
            "Rotate", "rotate", 'R', 0));
		add (net.grelf.Util.menuItem (this,
            "Translate", "translate", 'T', 0));
    add (net.grelf.Util.menuItem (this,
            "Swap quadrants (FFT)", "quadrants", 'Q', 0));
		addSeparator ();
		add (net.grelf.Util.menuItem (this,
            "Inverse gnomonic (plane to sphere)", "gnomonic_inv", 'I', 0));
		add (net.grelf.Util.menuItem (this,
            "Gnomonic projection (sphere to plane)", "gnomonic", 'G', 0));
		add (net.grelf.Util.menuItem (this,
            "Make warping grid", "warpmake", 'M', 0));
		add (net.grelf.Util.menuItem (this,
            "Apply warping grid", "warpuse", 'W', 0));
		addSeparator ();
		add (net.grelf.Util.menuItem (this,
            "Help about this menu", "help", 0, 0));
	} // initialise

	/** Implementing ImageSizeAction */
	@Override
	public void process (ImageSizeDialogue isd)
	{
		int width = isd.getWd ();
		int height = isd.getHt ();
		isd.dispose ();

		if (0 < width && 0 < height)
		{
			imframe.fit (width, height);
			imframe.addHistory ("Fit in " + width + " x " + height);
		}
	} // process

	/** Implementing the ActionListener interface to act on menu option selection. */
	@Override
	public void actionPerformed (java.awt.event.ActionEvent ev)
	{
		String action = ev.getActionCommand ();

		if (action.equals ("autocrop"))
		{
			imframe.addHistory ("Crop automatically");
			imframe.autoCrop ();
		}
		else
		if (action.equals ("crop"))
		{
/*
			boolean ok = true;

			if (null != imframe.getBlobMeases ())
			{
				ok = Util.confirm ("Invalidate blob data?", "Cropping will invalidate blob data\nexisting for this image.\nOK to proceed?");
			}

			if (ok)
*/
			{
				imframe.addHistory ("Crop manually");
				imframe.pushCaption ();
				imframe.getGP ().setDrawingMode (DrawingMode.CROP);
				imframe.setTitle ("Click to set rectangle corners (Ctrl for square)");
				// The end of the cropping process is then in Image.crop ().
			}
		}
		else
		if (action.equals ("croppower2"))
		{
      imframe.cropPower2 ();
    }
		else
		if (action.equals ("expandpower2"))
		{
      imframe.expandPower2 ();
    }
		else
		if (action.equals ("fit"))
		{
			int width = 1024;
			int height = 768;
			new ImageSizeDialogue (width, height, "Scale to fit", this);
			// Will call back to GeometryMenu.process ()
		}
		else
		if (action.equals ("fliph"))
		{
			imframe.addHistory ("Flip horizontally");
			imframe.flipHorizontal ();
		}
		else
		if (action.equals ("flipv"))
		{
			imframe.addHistory ("Flip vertically");
			imframe.flipVertical ();
		}
		else
		if (action.equals ("gnomonic"))
		{
			imframe.addHistory ("Gnomonic projection");
		  imframe.gnomonicProjection ();
		}
		else
		if (action.equals ("gnomonic_inv"))
		{
			imframe.addHistory ("Inverse gnomonic projection");
		  imframe.inverseGnomonicProjection ();
		}
		else
		if (action.equals ("help"))
		{
			HelpMenu.help ("pages/geom_menu.html");
		}
		else
		if (action.equals ("margin"))
		{
			imframe.addHistory ("Remove RAW margin");
			Raw.removeMargin (imframe);
			itemRaw.setEnabled (false);
		}
    else
    if (action.equals ("quadrants"))
    {
      imframe.swapQuadrants ();
    }
		else
		if (action.equals ("rotate"))
		{
			imframe.addHistory ("Rotate");
			imframe.rotate ();
		}
		else
		if (action.equals ("translate"))
		{
			imframe.addHistory ("Translate");
			imframe.translate ();
		}
		else
		if (action.equals ("scale"))
		{
			imframe.addHistory ("Scale");
			imframe.scale ();
		}
    else
    if (action.equals ("warpmake"))
    {
      if (null != WarpingGrid.make (imframe))
      {
        imframe.addHistory ("made warping grid");
      }
    }
    else
    if (action.equals ("warpuse"))
    {
      if (null != WarpingGrid.use (imframe))
      {
        imframe.addHistory ("used warping grid");
      }
    }
	} // actionPerformed

	/** Implementing MenuListener. */
	@Override
	public void menuSelected (javax.swing.event.MenuEvent ev)
	{
	} // menuSelected

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuDeselected (javax.swing.event.MenuEvent ev) {}

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuCanceled (javax.swing.event.MenuEvent ev) {}

} // GeometryMenu