package net.grelf.grip;

// This Java source file is part of the application GRIP, for processing and
// measuring digital photographs.
// GRIP is available free from www.grelf.net - please always quote this URL in
// connection with GRIP.
//
// Copyright (c) Graham T Relf, UK, 2006
//
// All software contains bugs, even if, we hope, they are only extremely subtle
// ones. The author has tested this software to the best of his abilities to
// ensure that it is generally fit for its purpose of processing and measuring
// digital photographs. The author makes absolutely no claims and gives no
// guarantees whatsoever as to the accuracy of any output from this software.
// Anyone making decisions based on any output from this software has a
// responsibility to first convince themselves of the appropriateness of so
// doing. They should test the software for themselves and understand how it
// works, sufficiently for their needs. Source code and explanatory
// documentation are available for that purpose.

/** File menu for the main GRIP frame, and actions. */
public class GripFileMenu
extends javax.swing.JMenu
implements java.awt.event.ActionListener, javax.swing.event.MenuListener,
           ImageSizeAction
{
	private javax.swing.JMenuItem itemImage, itemBlobs, itemChart, itemAAVSO;

	public GripFileMenu ()
	{
		super ("File");
		setMnemonic ('F');
    initialise ();
  } // GripFileMenu

  private void initialise ()
  {
    add (itemImage = net.grelf.Util.menuItem (this,
            "Open image(s)", "open",
            'O', java.awt.event.KeyEvent.VK_O));
		itemImage.setToolTipText ("Load RAW, TIFF, JPEG, PNG, etc");
    add (net.grelf.Util.menuItem (this,
            "New blank image", "new",
            'N', java.awt.event.KeyEvent.VK_N));
    add (net.grelf.Util.menuItem (this,
            "New star chart", "star",
            'C', 0));
		add (net.grelf.Util.menuItem (this,
            "New gradient chart", "gradient",
            'G', 0));
		add (net.grelf.Util.menuItem (this,
            "New test grid of squares", "grid",
            'D', 0));
		add (net.grelf.Util.menuItem (this,
            "New Mandelbrot curve", "mandel",
            'M', 0));
    
    if (GRIP.INCLUDE_EXPERIMENTS)
    {
      add (net.grelf.Util.menuItem (this,
            "New drawing", "drawing",
            'W', 0));
    }
    
		addSeparator ();
    add (itemBlobs = net.grelf.Util.menuItem (this,
            "Open blob data & image", "blob",
            'B', 0));
		itemBlobs.setToolTipText ("Load a .blobs file saved by GRIP");
    add (itemChart = net.grelf.Util.menuItem (this,
            "Open star chart", "chart",
            'S', 0));
		itemChart.setToolTipText ("Load a .chart file saved by GRIP");
    add (itemAAVSO = net.grelf.Util.menuItem (this,
            "Open AAVSO sequence", "aavso",
            'A', 0));
		itemAAVSO.setToolTipText ("Load saved HTML file as a chart");
    addSeparator ();
		add (net.grelf.Util.menuItem (this,
            "Find and refresh image table", "table",
            'T', java.awt.event.KeyEvent.VK_T));
    addSeparator ();
    add (net.grelf.Util.menuItem (this, "Exit", "exit",
            'X', 0));
	} // initialise

	/** Implementing ImageSizeAction */
	@Override
	public void process (ImageSizeDialogue isd)
	{
		int width = isd.getWd ();
		int height = isd.getHt ();
		isd.dispose ();

		if (0 < width && 0 < height)
		{
			GRIP.incNNewImages ();
			new ImFrame ("New image " + GRIP.getNNewImages (), GRIP.getNNewImages (),
              width, height, new DefaultMeasurementHandler ());
		}
	} // process

	/** Implementing the ActionListener interface to act on menu option selection. */
	@Override
	public void actionPerformed (java.awt.event.ActionEvent ev)
	{
		String action = ev.getActionCommand ();

    switch (action)
    {
      case "aavso":
        GRIP.getInstance ().openAAVSO_Chart ();
        break;

      case "blob":
        GRIP.getInstance ().openBlobs ();
        break;

      case "chart":
        GRIP.getInstance ().openChart ();
        break;

      case "drawing":
        new net.grelf.draw.DrawingFrame ("Drawing", 4000, 3000);
        break;

      case "exit":
        GRIP.getInstance ().exit ();
        break;

      case "gradient":
        new ImGraphic ().drawGradient ();
        break;

      case "grid":
        new ImGraphic ().drawGrid ();
        break;

      case "mandel":
        new ImGraphic ().drawMandelbrot ();
        break;

      case "new":
        int width = 1024;
        int height = 768;
        new ImageSizeDialogue (width, height, "New image", this);
        break;

      case "open":
        GRIP.getInstance ().openFiles ();
        break;

      case "star":
        net.grelf.astro.StarChart.createChart ();
        break;

      case "table":
        ImTable table = GRIP.getTable ();

        if (null != table)
        {
          table.refresh ();
          table.setVisible (true);
        }
        else

          if (0 < GRIP.getImFramesList ().size ())
        {
          GRIP.setTable (new ImTable (GRIP.getInstance (), "Select images", true));
        }
        else
        {
          net.grelf.Util.message (this, "Sorry", "No images currently open");
        }

        break;
    }
	} // actionPerformed

	/** Implementing MenuListener. */
	@Override
	public void menuSelected (javax.swing.event.MenuEvent ev)
	{
	} // menuSelected

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuDeselected (javax.swing.event.MenuEvent ev) {}

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuCanceled (javax.swing.event.MenuEvent ev) {}

} // GripFileMenu