package net.grelf.grip;

// This Java source file is part of the application GRIP, for processing and
// measuring digital photographs.
// GRIP is available free from www.grelf.net - please always quote this URL in
// connection with GRIP.
//
// Copyright (c) Graham T Relf, UK, 2006
//
// All software contains bugs, even if, we hope, they are only extremely subtle
// ones. The author has tested this software to the best of his abilities to
// ensure that it is generally fit for its purpose of processing and measuring
// digital photographs. The author makes absolutely no claims and gives no
// guarantees whatsoever as to the accuracy of any output from this software.
// Anyone making decisions based on any output from this software has a
// responsibility to first convince themselves of the appropriateness of so
// doing. They should test the software for themselves and understand how it
// works, sufficiently for their needs. Source code and explanatory
// documentation are available for that purpose.

/** This class provides the menu bar for class ImTable. */
public class ImTableMenu
implements java.awt.event.ActionListener, javax.swing.event.MenuListener,
           javax.swing.event.ListSelectionListener
{
	private javax.swing.JFrame frame;
	private ImTable table;

	private javax.swing.JMenuBar menubar;
	public javax.swing.JMenuBar getMenuBar () { return menubar; }

	private javax.swing.JMenuItem itemSaveCSV, itemSaveXML, itemCloseSel,
                                itemFFT, itemInverseFFT, itemLogMod;
	private javax.swing.JMenuItem itemSelect, itemDeselect, itemFront,
                                itemRefresh, itemSwap, itemCamera;
	private javax.swing.JMenuItem itemAdd, itemAddP, itemAve, itemDiv,
                                itemRevSub, itemSub, itemSubZero, itemMpy;
  private javax.swing.JMenuItem itemMpyP, itemReapply, itemRecombBands,
                                itemStereo, itemWarp, itemRGB,
                                itemKeepDiffer;

  private static final String CRITERION_2 =
          "Must be 2 selected, both same bit depth";
  private static final String CRITERION_28 =
          "Must be 2 selected, both 8 bits per channel";
  private static final String CRITERION_2MORE =
          "Must be 2 or more selected, all same bit depth";
  private static final String CRITERION_2I =
          "Must be 2 selected, both same bit depth, 1 mono & other multi-band";
  private static final String CRITERION_FFT =
          "Must be 2 selected, Re/Im parts of FFT (shown in metadata)";
  private static final String CRITERION_3 =
          "Must be 3 selected, all same bit depth";
  private static final String CRITERION_3MONO =
          "Must be 3 selected, all monochrome";

  private javax.swing.JMenu fileMenu;

	public ImTableMenu (javax.swing.JFrame aFrame, ImTable aTable)
	{
    this.frame = aFrame;
    this.table = aTable;
    this.fileMenu = new javax.swing.JMenu ("File");
    this.fileMenu.setMnemonic ('F');
    initialise ();
  } // ImTableMenu

  private void initialise ()
  {
    fileMenu.add (itemSaveCSV =
            net.grelf.Util.menuItem (this,
            "Save table as CSV", "savecsv", 'S', 0));
    fileMenu.add (itemSaveXML =
            net.grelf.Util.menuItem (this,
            "Save table as XML", "savexml", 'X',
                  java.awt.event.KeyEvent.VK_S));
		fileMenu.addSeparator ();
		fileMenu.add (itemFront =
            net.grelf.Util.menuItem (this,
            "Show selected in front", "front", 'F',
		              java.awt.event.KeyEvent.VK_F5));
    fileMenu.add (itemCloseSel =
            net.grelf.Util.menuItem (this,
            "Close selected images", "closeSel", 'C', 0));
		fileMenu.addSeparator ();
		fileMenu.add (
            net.grelf.Util.menuItem (this,
            "Find GRIP window", "grip", 'G',
              java.awt.event.KeyEvent.VK_G));
/* Removed 9.7.25
		fileMenu.add (itemClose =net.grelf.Util.menuItem (this,
          "Close this window", "close", 'W', java.awt.event.KeyEvent.VK_W));
*/

    javax.swing.JMenu editMenu = new javax.swing.JMenu ("Edit");
    editMenu.setMnemonic ('E');
		editMenu.add (itemSelect =
            net.grelf.Util.menuItem (this,
            "Select all", "select", 'S',
                  java.awt.event.KeyEvent.VK_A));
		editMenu.add (itemDeselect =
            net.grelf.Util.menuItem (this,
            "Deselect all", "deselect", 'D',
                  java.awt.event.KeyEvent.VK_D));
		editMenu.addSeparator ();
		editMenu.add (itemSwap =
            net.grelf.Util.menuItem (this,
            "Swap 2 selected images", "swap", 'W', 0));
		editMenu.addSeparator ();
		editMenu.add (itemRefresh =
            net.grelf.Util.menuItem (this,
            "Refresh the table", "refresh", 'R',
                  java.awt.event.KeyEvent.VK_F5));

    javax.swing.JMenu combineMenu = new javax.swing.JMenu ("Combine");
    combineMenu.setMnemonic ('C');

		combineMenu.add (itemAdd =
            net.grelf.Util.menuItem (this,
            "Add selected images",
                     "add", 'A', 0));
    itemAdd.setToolTipText (CRITERION_2MORE);

    combineMenu.add (itemAddP =
            net.grelf.Util.menuItem (this,
            "Add proportions of selected images",
                     "addP", 'P', 0));
    itemAddP.setToolTipText (CRITERION_2);

    combineMenu.add (itemAve =
            net.grelf.Util.menuItem (this,
            "Average selected images",
                     "ave", 'V', 0));
    itemAve.setToolTipText (CRITERION_2MORE);

    combineMenu.add (itemSub =
            net.grelf.Util.menuItem (this,
            "Subtract selected images (to grey)",
                     "sub", 'S', 0));
    itemSub.setToolTipText (CRITERION_2);

		combineMenu.add (itemSubZero =
            net.grelf.Util.menuItem (this,
            "Subtract selected images (to zero)",
                     "subzero", 'Z', 0));
    itemSubZero.setToolTipText (CRITERION_2);

		combineMenu.add (itemRevSub =
            net.grelf.Util.menuItem (this,
            "Reverse subtract selected images (to grey)",
                     "revsub", 'R', 0));
    itemRevSub.setToolTipText (CRITERION_2);

		combineMenu.add (itemMpy =
            net.grelf.Util.menuItem (this,
            "Multiply selected images",
                     "mpy", 'M', 0));
    itemMpy.setToolTipText (CRITERION_2);

		combineMenu.add (itemMpyP =
            net.grelf.Util.menuItem (this,
            "Multiply proportions of selected images",
                     "mpyP", 'O', 0));
    itemMpyP.setToolTipText (CRITERION_2);

    combineMenu.add (itemDiv =
            net.grelf.Util.menuItem (this,
            "Divide 1st image by 2nd (flat field)",
                     "div", 'D', 0));
    itemDiv.setToolTipText (CRITERION_2);

    combineMenu.add (itemKeepDiffer =
            net.grelf.Util.menuItem (this,
            "Keep pixels which differ from 2nd image",
                     "keep", 'K', 0));

    combineMenu.add (itemFFT =
            net.grelf.Util.menuItem (this,
            "Fast Fourier Transform (FFT)",
                     "fft", 'F', 0));
    itemDiv.setToolTipText (CRITERION_FFT);

    combineMenu.add (itemInverseFFT =
            net.grelf.Util.menuItem (this,
            "Inverse FFT", "fftInverse", 'I', 0));
    itemDiv.setToolTipText (CRITERION_FFT);

    combineMenu.add (itemLogMod =
            net.grelf.Util.menuItem (this,
            "View log mod (FFT pair)",
                     "fftLogMod", 'L', 0));
    itemDiv.setToolTipText (CRITERION_FFT);

		combineMenu.add (itemReapply =
            net.grelf.Util.menuItem (this,
            "Reapply colour",
                     "reapply", 'Y', 0));
    itemReapply.setToolTipText (CRITERION_2I);

		combineMenu.add (itemRecombBands =
            net.grelf.Util.menuItem (this,
            "Recombine channels",
                     "recomb", 'C', 0));
    itemRecombBands.setToolTipText (CRITERION_3);

    combineMenu.add (itemRGB =
            net.grelf.Util.menuItem (this,
            "Build 1 RGB image",
                     "rgb", '1', 0));
    itemRGB.setToolTipText (CRITERION_3MONO);

		combineMenu.add (itemStereo =
            net.grelf.Util.menuItem (this,
            "Show as red-green stereo",
                     "stereo", 'T', 0));
    itemStereo.setToolTipText (CRITERION_28);

		combineMenu.add (itemWarp = net.grelf.Util.menuItem (this,
                     "Adjust control points and warp",
                     "warp", 'W', 0));

    combineMenu.add (itemCamera = net.grelf.Util.menuItem (this,
                       "Compute camera response curve",
                       "camera", 'E', 0));

    combineMenu.addSeparator ();
		combineMenu.add (net.grelf.Util.menuItem (this,
            "Help about this menu",
                     "help", 'H', 0));

    menubar = new javax.swing.JMenuBar ();
    menubar.add (fileMenu);
    menubar.add (editMenu);
    menubar.add (combineMenu);
    menubar.add (new HelpMenu ());

		table.getJTable ().getSelectionModel ().addListSelectionListener (this);
		enableItems ();
    frame.setJMenuBar (menubar);
  } // initialise

	protected void enableItems ()
	{
    boolean allSameBitDepth = areSelectedAllSameBitDepth ();

		itemSaveCSV.setEnabled (true);
		itemSaveXML.setEnabled (true);
		itemCloseSel.setEnabled (true);
		itemSelect.setEnabled (true);
		itemDeselect.setEnabled (true);
    itemCamera.setEnabled (allSameBitDepth);

    // Disable the rest until we can prove they are suitable:
		itemFront.setEnabled (false);
		itemSwap.setEnabled (false);
		itemRefresh.setEnabled (true);
		itemAdd.setEnabled (false);
		itemAddP.setEnabled (false);
		itemAve.setEnabled (false);
		itemDiv.setEnabled (false);
		itemRevSub.setEnabled (false);
		itemSub.setEnabled (false);
		itemSubZero.setEnabled (false);
		itemMpy.setEnabled (false);
		itemMpyP.setEnabled (false);
    itemFFT.setEnabled (false);
    itemInverseFFT.setEnabled (false);
    itemReapply.setEnabled (false);
    itemRGB.setEnabled (false);
		itemRecombBands.setEnabled (false);
		itemStereo.setEnabled (false);
		itemWarp.setEnabled (false);
    itemKeepDiffer.setEnabled (false);

		int nSelected = table.getNSelected ();

    switch (nSelected)
		{
    case 0:
      itemCloseSel.setEnabled (false);
      itemDeselect.setEnabled (false);
      itemCamera.setEnabled (false);
      break;

    case 1:
      itemFront.setEnabled (true);
      itemCamera.setEnabled (false);
      break;

    case 2:
      if (allSameBitDepth)
      {
        itemAdd.setEnabled (true);
        itemAddP.setEnabled (true);
        itemAve.setEnabled (true);
        itemDiv.setEnabled (true);
        itemKeepDiffer.setEnabled (true);
        itemRevSub.setEnabled (true);
        itemSub.setEnabled (true);
        itemSubZero.setEnabled (true);
        itemMpy.setEnabled (true);
        itemMpyP.setEnabled (true);
        itemStereo.setEnabled (true);
        itemSwap.setEnabled (true);
        itemFFT.setEnabled (true);
        itemInverseFFT.setEnabled (true);

        java.util.List <ImFrame> selected = table.getSelected ();
        net.grelf.image.Image image0 = selected.get (0).getImage ();
        net.grelf.image.Image image1 = selected.get (1).getImage ();
        int nBands0 = image0.getNBands ();
        int nBands1 = image1.getNBands ();

        if ((1 == nBands0 && 1 < nBands1)
         || (1 < nBands0 && 1 == nBands1))
        {
          itemReapply.setEnabled (true);
        }
/*
        GlassPane gp1 = selected.get (0).getImPane ().getGlassPane ();
        GlassPane gp2 = selected.get (1).getImPane ().getGlassPane ();

        if (DrawingMode.CONTROL_POINTS == gp1.getDrawingMode ()
         && DrawingMode.CONTROL_POINTS == gp2.getDrawingMode ())
        {*/
          itemWarp.setEnabled (true);
        /*}
*/
      }

      break;

    case 3:
      if (allSameBitDepth)
      {
        itemAdd.setEnabled (true);
        itemAve.setEnabled (true);
        itemRecombBands.setEnabled (true);

        boolean allSingleChannel = true; // Until shown false

        for (ImFrame imf : table.getSelected ())
        {
          if (imf.getImage ().getNBands () != 1)
          {
            allSingleChannel = false;
          }
        }

        itemRGB.setEnabled (allSingleChannel);
      }

      break;

    default:
      if (allSameBitDepth)
      {
        itemAdd.setEnabled (true);
        itemAve.setEnabled (true);
      }
		}
	} // enableItems

  /** Only called if number selected is known to be greater than 1. */
  private boolean areSelectedAllSameBitDepth ()
  {
    java.util.List <ImFrame> selected = this.table.getSelected ();

    if (null == selected || 0 == selected.size ()) { return false; }

    int bits0 = selected.get (0).getImage ().getBitsPerChannel ();

    for (int i = 1; i < selected.size (); i++)
    {
      if (bits0 != selected.get (i).getImage ().getBitsPerChannel ())
      {
        return false;
      }
    }

    return true;
  } // areSelectedAllSameBitDepth

	/** Implementing ListSelectionListener, to respond to the user selecting or
    * deselecting rows. */
	@Override
	public void valueChanged (javax.swing.event.ListSelectionEvent ev)
	{
		if (ev.getValueIsAdjusting ()) { return; }

		enableItems ();
	} // valueChanged

	/** Implementing ActionListener, to respond to menu option selection. */
	@Override
	public void actionPerformed (java.awt.event.ActionEvent ev)
	{
		String action = ev.getActionCommand ();

    switch (action)
    {
      case "add":
        if (2 > table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select at least 2 images, to add them");
        }
        else
        {
          table.combine ("add");
        }
        break;

      case "addP":
        if (2 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select 2 images, to add them proportionately");
        }
        else
        {
          table.combine ("addP");
        }
        break;

      case "ave":
        if (2 > table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select at least 2 images, to average them");
        }
        else
        {
          table.combine ("ave");
        }
        break;

      case "camera":
        {
          java.util.List <ImFrame> selected = table.getSelected ();

          if (2 > selected.size ())
          {
            net.grelf.Util.warning (frame, "Sorry",
                  "Camera response curve needs at least 2 selected images");
          }
          else
          {
            if (! areSelectedAllSameBitDepth ())
            {
              net.grelf.Util.warning (frame, "Sorry",
                  "Camera response curve needs 2 images of the same bit depth");
            }
            else
            {
              ImFrame.showCameraResponseCurve (selected);
            }
          }
        }

        break;

      case "closeSel":
        {
          java.util.List <ImFrame> selected = table.getSelected ();
          for (int i = selected.size () - 1; i >= 0; i--)
          {
            ImFrame imf = selected.get (i);
            imf.setVisible (false);
            imf.dispose ();
          }
          table.refresh ();
          break;
        }

      case "deselect":
        table.getJTable ().getSelectionModel ().clearSelection ();
        break;

      case "div":
        if (2 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
"You must select exactly 2 images, to divide them with the 2nd being a flat field");
        }
        else
        {
          table.combine ("div");
        }
        break;

      case "fft":
        table.combine ("fft");
        break;

      case "fftInverse":
        table.combine ("fftInverse");
        break;

      case "fftLogMod":
        table.combine ("fftLogMod");
        break;

      case "front":
        {
          java.util.List<ImFrame> selected = table.getSelected ();
          if (1 == selected.size ())
          {
            ImFrame imf = selected.get (0);
            imf.toFront ();
          }
          break;
        }

      case "grip":
        GRIP.getInstance ().toFront ();
        break;

      case "help":
        HelpMenu.help ("pages/combine_menu.html");
        break;

      case "keep":
        if (2 > table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select exactly 2 images, to keep differing pixels");
        }
        else
        {
          table.combine ("keep");
        }
        break;

      case "mpy":
        if (2 > table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select exactly 2 images, to multiply them");
        }
        else
        {
          table.combine ("mpy");
        }
        break;

      case "mpyP":
        if (2 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select 2 images, to multiply them proportionately");
        }
        else
        {
          table.combine ("mpyP");
        }
        break;

      case "reapply":
        table.reapplyColour ();
        break;

      case "recomb":
        if (3 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
           "You must select exactly 3 images, to recombine channels from them");
        }
        else
        {
          table.recombine ();
        }
        break;

      case "refresh":
        table.refresh ();
        table.getJTable ().getSelectionModel ().addListSelectionListener (this);
        enableItems ();
        break;

      case "revsub":
        if (2 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select exactly 2 images, to subtract them");
        }
        else
        {
          table.combine ("revsub");
        }
        break;

      case "rgb":
        if (3 == table.getNSelected ())
        {
          java.util.List <ImFrame> selected = table.getSelected ();
          int nBits0 = selected.get (0).getImage ().getBitsPerChannel ();
          int nBits1 = selected.get (1).getImage ().getBitsPerChannel ();
          int nBits2 = selected.get (2).getImage ().getBitsPerChannel ();

          if (nBits0 != nBits1 || nBits0 != nBits2)
          {
            net.grelf.Util.message ("Sorry",
       "All 3 selected images must have the same number of bits per channel");
          }
          else
          {
            if (nBits0 == 64) // Image64
            {
              net.grelf.image.Image [] images = new net.grelf.image.Image [3];
              int [] widths = new int [3];
              int [] heights = new int [3];
              int i = 0;
              int width = 0, height = 0;

              for (ImFrame imf : selected)
              {
                images [i] = imf.getImage ();
                widths [i] = images [i].getWidth ();
                heights [i] = images [i].getHeight ();
                width = Math.max (width, widths [i]);
                height = Math.max (height, heights [i]);
                i++;
              }

              net.grelf.image.Image64 dstImage =
                      new net.grelf.image.Image64 (width, height, 3);

              if (null != dstImage)
              {
                double [] wpx = new double [3];
                double g;

                for (int y = 0; y < height; y++)
                {
                  for (int x = 0; x < width; x++)
                  {
                    for (int b = 0; b < 3; b++)
                    {
                      if (x < widths [b] && y < heights [b])
                      {
                        g = images [b].getPixelDouble (x, y) [0];
                        wpx [b] = g;
                      }
                      else
                      {
                        wpx [b] = 0;
                      }
                    }

                    dstImage.setPixelDouble (x, y, wpx);
                  }
                }

                dstImage.getRangeByRescanningDouble ();
                ImFrame imf = new ImFrame ("RGB", dstImage);
                imf.redisplay ();
              }
            }
            else // nBits == 8, 16 or 32:
            {
              net.grelf.image.Image [] images = new net.grelf.image.Image [3];
              int [] widths = new int [3];
              int [] heights = new int [3];
              int i = 0;
              int width = 0, height = 0;

              for (ImFrame imf : selected)
              {
                images [i] = imf.getImage ();
                widths [i] = images [i].getWidth ();
                heights [i] = images [i].getHeight ();
                width = Math.max (width, widths [i]);
                height = Math.max (height, heights [i]);
                i++;
              }

              net.grelf.image.Image dstImage;

              if (nBits0 == 8)
              {
                dstImage = new net.grelf.image.Image8 (width, height, 3);
              }
              else
              if (nBits0 == 16)
              {
                dstImage = new net.grelf.image.Image16 (width, height, 3);
              }
              else
              {
                dstImage= new net.grelf.image.Image32 (width, height, 3);
              }

              if (null != dstImage)
              {
                int [] wpx = new int [3];
                int g;

                for (int y = 0; y < height; y++)
                {
                  for (int x = 0; x < width; x++)
                  {
                    for (int b = 0; b < 3; b++)
                    {
                      if (x < widths [b] && y < heights [b])
                      {
                        g = images [b].getPixel (x, y) [0];
                        wpx [b] = g;
                      }
                      else
                      {
                        wpx [b] = 0;
                      }
                    }

                    dstImage.setPixel (x, y, wpx);
                  }
                }

                dstImage.getRangeByRescanning ();
                ImFrame imf = new ImFrame ("RGB", dstImage);
                imf.redisplay ();
              }
            }
          }
        }
        break;

      case "savecsv":
        table.saveAsCSV ();
        break;

      case "savexml":
        table.saveAsXML ();
        break;

      case "select":
        table.getJTable ().getSelectionModel ().
                setSelectionInterval (0,
                table.getJTable ().getModel ().getRowCount () - 1);
        break;

      case "stereo":
        if (2 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select exactly 2 images, to show them in stereo");
        }
        else
        {
          table.combine ("stereo");
        }
        break;

      case "sub":
        if (2 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select exactly 2 images, to subtract them");
        }
        else
        {
          table.combine ("sub");
        }
        break;

      case "subzero":
        if (2 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select exactly 2 images, to subtract them");
        }
        else
        {
          table.combine ("subzero");
        }
        break;

      case "swap":
        if (2 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                  "You must select exactly 2 images, to swap them");
        }
        else
        {
          table.swap ();
        }
        break;

      case "warp":
				if (2 != table.getNSelected ())
        {
          net.grelf.Util.warning (frame, "Not possible",
                       "You must select exactly 2 images, to warp them");
        }
        else
        {
          table.combine ("warp");
        }

        break;
    }

		table.refresh ();
		enableItems ();
	} // actionPerformed

	/** Implementing MenuListener. */
	@Override
	public void menuSelected (javax.swing.event.MenuEvent ev)
	{
    enableItems ();
	} // menuSelected

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuDeselected (javax.swing.event.MenuEvent ev) {}

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuCanceled (javax.swing.event.MenuEvent ev) {}

} // ImTableMenu