package net.grelf.grip;

// This Java source file is part of the application GRIP, for processing and
// measuring digital photographs.
// GRIP is available free from www.grelf.net - please always quote this URL in
// connection with GRIP.
//
// Copyright (c) Graham T Relf, UK, 2006
//
// All software contains bugs, even if, we hope, they are only extremely subtle
// ones. The author has tested this software to the best of his abilities to
// ensure that it is generally fit for its purpose of processing and measuring
// digital photographs. The author makes absolutely no claims and gives no
// guarantees whatsoever as to the accuracy of any output from this software.
// Anyone making decisions based on any output from this software has a
// responsibility to first convince themselves of the appropriateness of so
// doing. They should test the software for themselves and understand how it
// works, sufficiently for their needs. Source code and explanatory
// documentation are available for that purpose.

/** Image menu (on each ImFrame) and its actions. */
public class ImageMenu extends javax.swing.JMenu
implements java.awt.event.ActionListener, javax.swing.event.MenuListener
{
	private ImFrame imframe;
	private javax.swing.JMenuItem itemPseudo, itemBlobs, itemChart, itemUndo;

	public ImageMenu (ImFrame frame)
	{
		super ("Image");
		imframe = frame;
    setMnemonic ('I');
    initialise ();
  } // ImageMenu

  private void initialise ()
  {
    add (net.grelf.Util.menuItem (this,
            "Zoom in", "zoomin", '=', java.awt.event.KeyEvent.VK_EQUALS));
    add (net.grelf.Util.menuItem (this,
            "Zoom out", "zoomout", '-', java.awt.event.KeyEvent.VK_MINUS));
    add (net.grelf.Util.menuItem (this,
            "Fit to screen", "zoomfit", 'F', java.awt.event.KeyEvent.VK_0));
		add (itemPseudo = net.grelf.Util.menuItem (this,
            "Display in pseudocolour", "pseudo", 'P', 0));
		itemPseudo.setToolTipText ("Affects display only, not the image");
		addSeparator ();
    add (net.grelf.Util.menuItem (this,
            "Save as", "saveas", 'S', java.awt.event.KeyEvent.VK_S));
    add (itemBlobs = net.grelf.Util.menuItem (this,
            "Save with blob/star data", "saveblobs", 'B', 0));
		itemBlobs.setToolTipText ("Saves 2 files: image and .blobs (XML)");
    add (itemChart = net.grelf.Util.menuItem (this,
            "Save as star chart", "savechart", 'V', 0));
		itemChart.setToolTipText ("Saves XML; window caption is chart title");
		addSeparator ();
    add (itemUndo = net.grelf.Util.menuItem (this,
            "Undo (not)", "undo", 'U', java.awt.event.KeyEvent.VK_Z));
    itemUndo.setToolTipText ("Select for an explanation of why there is no Undo");
		add (net.grelf.Util.menuItem (this,
            "Clone, in new window", "clone", 'C', 0));
		add (net.grelf.Util.menuItem (this,
            "Reload this window from file", "reload", 'R', 0));
		addSeparator ();
    add (net.grelf.Util.menuItem (this,
            "Show image information", "info", 'I', 0));
		add (net.grelf.Util.menuItem (this,
            "View history of this window", "history", 'H', 0));
		add (net.grelf.Util.menuItem (this,
            "Edit window caption", "caption", 'E', 0));
		add (net.grelf.Util.menuItem (this,
            "Add 1-pixel white rim", "rim", '1', 0));
		addSeparator ();
		add (net.grelf.Util.menuItem (this,
            "Find GRIP window", "grip", 'G', java.awt.event.KeyEvent.VK_G));
    add (net.grelf.Util.menuItem (this,
            "Close this window", "close", 'W', java.awt.event.KeyEvent.VK_W));
		addSeparator ();
		add (net.grelf.Util.menuItem (this,
            "Help about this menu", "help", 0, 0));
		addMenuListener (this);
	} // initialise

	private void reviseSettings ()
	{
    ImFrame imf = this.imframe;

		itemBlobs.setEnabled (null != imf.getBlobMeases ());

    if (imf instanceof StarChartFrame)
    {
  		itemChart.setEnabled (null != ((StarChartFrame) imf).getStarChart ());
    }
    else
    {
      itemChart.setEnabled (false);
    }
	} // reviseSettings

	@Override
	public void menuSelected (javax.swing.event.MenuEvent ev)
	{
		reviseSettings ();
	}

	/** Implementing MenuListener but does nothing */
	@Override
	public void menuDeselected (javax.swing.event.MenuEvent ev) {}

	/** Implementing MenuListener but does nothing */
	@Override
	public void menuCanceled (javax.swing.event.MenuEvent ev) {}

	/** Implementing the ActionListener interface to act on menu option selection. */
	@Override
	public void actionPerformed (java.awt.event.ActionEvent ev)
	{
		String action = ev.getActionCommand ();
    switch (action)
    {
      case "caption":
        String newCaption = net.grelf.Util.askString (
                "Image window caption", imframe.getCaption ());

        if (null != newCaption)
        {
          imframe.setCaption (newCaption);
        }

        break;

      case "clone":
        imframe.duplicate ();
        break;

      case "close":
        imframe.setVisible (false);
        imframe.dispose ();
        break;

      case "grip":
        GRIP.getInstance ().toFront ();
        break;

      case "help":
        HelpMenu.help ("pages/image_menu.html");
        break;

      case "history":
        imframe.showHistory ();
        break;

      case "info":
        imframe.getImage ().showInfo ();
        break;

      case "pseudo":
        ImPane ip = imframe.getImPane ();

        if (ip.isPseudocolour ())
        {
          ip.setPseudocolour (null);
          itemPseudo.setText ("Display in pseudocolour");
        }
        else
        {
          try
          {
            java.awt.image.BufferedImage scaledImage = ip.getScaledImage ();
            int nBits = net.grelf.image.Image8or16Base.getBitsPerChannel (scaledImage);
            int maxLevel = net.grelf.image.Image8or16Base.getMaxLevel (scaledImage);
            int ci = net.grelf.Util.askInteger (
                    "Contour interval (0 for no contours) [2.." + maxLevel + "]",
                    25, 0, maxLevel / 2);
            Pseudocolour ps;

            if (8 == nBits)
            {
              ps = new Pseudocolour (new net.grelf.image.Image8 (scaledImage));
            }
            else
            {
              ps = new Pseudocolour (new net.grelf.image.Image16 (scaledImage));
            }

            if (ci > 0)
            {
              ps.contour (ci);
            }

            ip.setPseudocolour (ps);
          }
          catch (net.grelf.image.IncompatibleImageException ex)
          {
            net.grelf.Util.warning ("Error", ex.getMessage ());
          }

          itemPseudo.setText ("Display without pseudocolour");
        }

        imframe.redisplay ();
        break;

      case "reload":
        String path = imframe.getFilePath ();

        if (null == path)
        {
          net.grelf.Util.message ("Sorry", "This image does not have an associated file");
        }
        else
        {
          imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
          imframe.addHistory ("Reload");
          net.grelf.image.Image image = net.grelf.image.ImageLoader.load (path);

          if (null == image)
          {
            net.grelf.Util.message ("Sorry", "Could not reload image from " + path);
          }
          else
          {
            imframe.getImPane ().setImage (image);
            imframe.redisplay ();
          }

          imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
        }

        break;

      case "rim":
        imframe.addHistory ("1-pixel white rim");
        new ImGraphic ().drawRim (imframe, 1, java.awt.Color.white);
        break;

      case "saveas":
        imframe.saveAs ();
        break;

      case "saveblobs":
        imframe.saveAs ();
        imframe.saveBlobMeasList ();
        break;

      case "savechart":
        if (imframe instanceof StarChartFrame)
        {
          String filepath = net.grelf.FileIO.selectOutputFile (
                  "Star chart", null,
                  new javax.swing.filechooser.FileFilter [] {
                      new net.grelf.FileIOChartFilter (
                              net.grelf.FileIOChartFilter.CHART) },
                      net.grelf.FileIO.removeExtension (
                              imframe.getCaption ()) + ".chart");

          if (null != filepath)
          {
            ((StarChartFrame) imframe).getStarChart ().saveAsXML (
                    filepath, imframe.getCaption ());
          }
        }

        break;

      case "undo":
        StringBuilder sb = new StringBuilder ("There is no Undo.\n");
        sb.append ("The reason for this is that we do not want to waste\n");
        sb.append ("processing time on making copies of images when\n");
        sb.append ("they may never be needed.\n");
        sb.append ("Instead we put the onus on you, the user. If you\n");
        sb.append ("think the next step you are about take may need\n");
        sb.append ("to be reversed, you have two options available:\n");
        sb.append ("(1) save the image on disc, so that Image/Reload\n");
        sb.append ("can bring it back quickly, or\n");
        sb.append ("(2) use Image/Clone to make a visible copy of the\n");
        sb.append ("image in a new window.\n");
        sb.append ("This is explained in the Help documentation,\n");
        sb.append ("along with much else.");
        net.grelf.Util.message (this.imframe, "Sorry", sb.toString ());
        break;

      case "zoomfit":
        imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
        imframe.getImPane ().setInitialScale ();
        imframe.redisplay ();
        imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
        break;

      case "zoomin":
        imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
        imframe.getImPane ().zoomUp ();
        imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
        break;

      case "zoomout":
        imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
        imframe.getImPane ().zoomDown ();
        imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
        break;
    }
	} // actionPerformed

} // ImageMenu