package net.grelf.grip;

// This Java source file is part of the application GRIP, for processing and
// measuring digital photographs.
// GRIP is available free from www.grelf.net - please always quote this URL in
// connection with GRIP.
//
// Copyright (c) Graham T Relf, UK, 2006
//
// All software contains bugs, even if, we hope, they are only extremely subtle
// ones. The author has tested this software to the best of his abilities to
// ensure that it is generally fit for its purpose of processing and measuring
// digital photographs. The author makes absolutely no claims and gives no
// guarantees whatsoever as to the accuracy of any output from this software.
// Anyone making decisions based on any output from this software has a
// responsibility to first convince themselves of the appropriateness of so
// doing. They should test the software for themselves and understand how it
// works, sufficiently for their needs. Source code and explanatory
// documentation are available for that purpose.

/** Levels menu (on each ImFrame) and its actions. */
public class LevelsMenu
extends javax.swing.JMenu
implements java.awt.event.ActionListener, javax.swing.event.MenuListener
{
	private ImFrame imframe;
	private javax.swing.JMenuItem itemRaw, itemDefect,
       item8bpc, item16bpc, item32bpc, item64bpc, itemMono, itemRGB,
       itemSplitIntensity, itemSplitChannels, itemInvert, itemMultiply,
       itemBackgr, itemPseudo, itemSaturate, itemEqualise, itemContour,
       itemStretch, itemCurves, itemBalance, itemNeutralise, itemHelp;

	private Pseudocolour lut = null;

	/** Enable/disable menu options according to the number of channels and bits
    * per channel in the currently displayed image and whether blob detection
    * has been done. */
	public void reviseSettings ()
	{
    net.grelf.image.Image image = this.imframe.getImage ();
    int nBits = image.getBitsPerChannel ();
    int nBands = image.getNBands ();

		if (image.isRaw ())
		{
      itemDefect.setEnabled (true);
      itemRaw.setEnabled (true);
      itemEqualise.setEnabled (false);
    }
    else
    {
      itemDefect.setEnabled (false);
      itemRaw.setEnabled (false);
      itemEqualise.setEnabled (true);
    }

    switch (nBits)
    {
      case 8:
        item8bpc.setEnabled (false);
        item16bpc.setEnabled (true);
        item32bpc.setEnabled (true);
        item64bpc.setEnabled (true);
        break;

      case 16:
        item8bpc.setEnabled (true);
        item16bpc.setEnabled (false);
        item32bpc.setEnabled (true);
        item64bpc.setEnabled (true);
        break;

      case 32:
        item8bpc.setEnabled (true);
        item16bpc.setEnabled (true);
        item32bpc.setEnabled (false);
        item64bpc.setEnabled (true);
        break;

      case 64:
        item8bpc.setEnabled (true);
        item16bpc.setEnabled (true);
        item32bpc.setEnabled (true);
        item64bpc.setEnabled (false);
        itemEqualise.setEnabled (false);
        break;
    }

    if (nBands == 1)
    {
      itemSaturate.setEnabled (false);
			itemSplitChannels.setEnabled (false);
      itemSplitIntensity.setEnabled (false);
      itemRGB.setEnabled (true);
    }
    else
    {
      itemSaturate.setEnabled (true);
			itemSplitChannels.setEnabled (true);
      itemSplitIntensity.setEnabled (true);
      itemRGB.setEnabled (false);
    }

		if (nBands == 3)
		{
			itemMono.setEnabled (true);
		}
		else
		{
			itemMono.setEnabled (false);
		}
	} // reviseSettings

	public LevelsMenu (ImFrame frame)
	{
		super ("Levels");
		this.imframe = frame;
    setMnemonic ('L');
    initialise ();
  } // LevelsMenu

  private void initialise ()
  {
    add (itemDefect = net.grelf.Util.menuItem (
            this, "Detect defective pixels...", "defects", 'F', 0));
    add (itemRaw = net.grelf.Util.menuItem (
            this, "Interpret RAW", "raw", 'W', 0));
    addSeparator ();

    javax.swing.JMenu bitsSubmenu = new javax.swing.JMenu ("Change bit depth");
		bitsSubmenu.add (item8bpc = net.grelf.Util.menuItem (
            this, "Convert to 8 bits per channel", "8bpc", '8', 0));
		bitsSubmenu.add (item16bpc = net.grelf.Util.menuItem (
            this, "Convert to 16 bits per channel", "16bpc", '1', 0));
		bitsSubmenu.add (item32bpc = net.grelf.Util.menuItem (
            this, "Convert to 32 bits per channel", "32bpc", '3', 0));
		bitsSubmenu.add (item64bpc = net.grelf.Util.menuItem (
            this, "Convert to 64 bits per channel", "64bpc", '6', 0));
    add (bitsSubmenu);

    add (itemMono = net.grelf.Util.menuItem (
            this, "Convert to monochrome", "mono", 'M', 0));
    add (itemRGB = net.grelf.Util.menuItem (
            this, "Convert to RGB", "rgb", 'R', 0));
		add (itemSplitIntensity = net.grelf.Util.menuItem (
            this, "Split intensity", "splitI", 'I', 0));
    itemSplitIntensity.setToolTipText (
            "Make mono intensity image plus colour image for later reapplying");
		add (itemSplitChannels = net.grelf.Util.menuItem (
            this, "Split channels", "split", 'C', 0));
    itemSplitChannels.setToolTipText ("Make a separate image for each channel");
		addSeparator ();

    add (itemInvert = net.grelf.Util.menuItem (
            this, "Invert", "invert", 'V', java.awt.event.KeyEvent.VK_I));
		add (itemStretch = net.grelf.Util.menuItem (
            this, "Auto-stretch", "autostretch", 'A', java.awt.event.KeyEvent.VK_L));
    add (itemMultiply = net.grelf.Util.menuItem (
            this, "Multiply", "multiply", 'Y', 0));
		add (itemBackgr = net.grelf.Util.menuItem (
            this, "Correct background", "background", 'G', 0));
    itemBackgr.setToolTipText ("Assumes background darker than foreground");
		add (itemCurves = net.grelf.Util.menuItem (
            this, "Curves", "curves", 'U', java.awt.event.KeyEvent.VK_M));
		add (itemEqualise = net.grelf.Util.menuItem (
            this, "Equalise histogram", "equalise", 'Q', 0));
		addSeparator ();

    add (itemSaturate = net.grelf.Util.menuItem (
            this, "Saturation", "saturation", 'S',
                                              java.awt.event.KeyEvent.VK_U));
		add (itemBalance = net.grelf.Util.menuItem (
            this, "Colour balance", "balance", 'L', 0));
		add (itemNeutralise = net.grelf.Util.menuItem (
            this, "Neutralise background", "neutralise", 'N', 0));
		add (itemPseudo = net.grelf.Util.menuItem (
            this, "Pseudocolour", "pseudo", 'P', 0));
		itemPseudo.setToolTipText ("Changes the image levels, not just the display");
		add (itemContour = net.grelf.Util.menuItem (
            this, "Contour", "contour", 'T', 0));
		addSeparator ();
		add (itemHelp = net.grelf.Util.menuItem (
            this, "Help about this menu", "help", 'E', 0));
		reviseSettings ();

		addMenuListener (this); // Need to be able to revise settings when the menu is invoked
	} // initialise

	/** Implementing the ActionListener interface to act on menu option selection. */
	@Override
	public void actionPerformed (java.awt.event.ActionEvent ev)
	{
		String action = ev.getActionCommand ();

    switch (action)
    {
      case "16bpc":
        {
          imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
          ImPane ip = imframe.getImPane ();
          net.grelf.image.Image image = ip.getImage ();
          ip.setImage (image.convertToImage16 ());
          imframe.redisplay ();
          imframe.addHistory ("Convert to 16 bits per channel");
          imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
          reviseSettings ();
          break;
        }

      case "32bpc":
        {
          imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
          ImPane ip = imframe.getImPane ();
          net.grelf.image.Image image = ip.getImage ();
          ip.setImage (image.convertToImage32 ());
          imframe.redisplay ();
          imframe.addHistory ("Convert to 32 bits per channel");
          imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
          reviseSettings ();
          break;
        }

      case "64bpc":
        {
          imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
          ImPane ip = imframe.getImPane ();
          net.grelf.image.Image image = ip.getImage ();
          ip.setImage (image.convertToImage64 ());
          imframe.redisplay ();
          imframe.addHistory ("Convert to 64 bits per channel");
          imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
          reviseSettings ();
          break;
        }

      case "8bpc":
        {
          imframe.setCursor (GRIP.GRIP_WAIT_CURSOR);
          ImPane ip = imframe.getImPane ();
          net.grelf.image.Image image = ip.getImage ();
          ip.setImage (image.convertToImage8 ());
          imframe.redisplay ();
          imframe.addHistory ("Convert to 8 bits per channel");
          imframe.setCursor (GRIP.GRIP_DEFAULT_CURSOR);
          reviseSettings ();
          break;
        }

      case "autostretch":
        imframe.addHistory ("Stretch levels automatically");
        imframe.autoStretch ();
        break;

      case "background":
        imframe.addHistory ("Correct background");
        imframe.correctBackground ();
        break;

      case "balance":
        ImColourDialogue.askSettings (imframe); // Adds history if not cancelled
        break;

      case "contour":
        ImContourDialogue.interact (imframe); // Adds history if not cancelled
        break;

      case "curves":
        ImCurveDialogue.askCurve (imframe); // Adds history if not cancelled
        break;

      case "defects":
        Config.defects = Raw.detectDefectivePixels (imframe);
        break;

      case "equalise":
        imframe.equaliseHistogram ();
        break;

      case "help":
        HelpMenu.help ("pages/level_menu.html");
        break;

      case "invert":
        imframe.addHistory ("Invert");
        imframe.invert ();
        break;

      case "mono":
        imframe.addHistory ("Convert to monochrome");
        int nb = imframe.getImage ().getNBands ();
        double [] f = new double [nb];

        for (int b = 0; b < nb; b++)
        {
          f [b] = 1.0;
        }

        imframe.convertToMonochrome (f);

        break;

      case "multiply":
        double factor = net.grelf.Util.askDouble (
                "Factor by which to multiply levels?", 1.0, 0.001, 1000.0);

        if (!Double.isNaN (factor))
        {
          imframe.multiply (factor);
        }

        break;

      case "neutralise":
        imframe.addHistory ("Neutralise background");
        imframe.neutraliseBackground ();
        break;

      case "pseudo":
        {
          imframe.addHistory ("Pseudocolour");
          net.grelf.image.Image image = imframe.getImage ();
          lut = new Pseudocolour (image);
          lut.apply (imframe);
          break;
        }

      case "rgb":
        imframe.addHistory ("Convert to RGB");
        
        if (1 == imframe.getImage ().getNBands ())
        {
          imframe.convertMonoToRGB ();
        }
        else
        {
          net.grelf.Util.warning ("ERROR", "Must be a monochrome image to convert");
        }
        
        break;

      case "saturation":
        ImSaturateDialogue.askSaturate (imframe); // Adds history if not cancelled
        break;

      case "split":
        imframe.addHistory ("Split channels");
        imframe.splitChannels ();
        break;

      case "splitI":
        imframe.addHistory ("Split intensity");
        imframe.splitIntensity ();
        break;

      case "raw":
        imframe.addHistory ("Interpret raw");
        Raw.interpret (imframe);
        itemRaw.setEnabled (false);
        itemDefect.setEnabled (false);
        break;
    }
	} // actionPerformed

	/** Calls reviseSettings (). Implementing MenuListener. */
	@Override
	public void menuSelected (javax.swing.event.MenuEvent ev)
	{
		reviseSettings ();
	} // menuSelected

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuDeselected (javax.swing.event.MenuEvent ev) {}

	/** Implementing MenuListener but does nothing. */
	@Override
	public void menuCanceled (javax.swing.event.MenuEvent ev) {}

} // LevelsMenu